### R code from vignette source 'ex8a4.Rnw'

###################################################
### code chunk number 1: ex8a4.Rnw:19-24
###################################################
options(prompt="> ", continue="+ ", digits=5, width=70, show.signif.stars=T)
par(mfrow=c(1,1))
## clean workspace
rm(list=ls())
setwd("~/data/edu/dgeostats/ex")


###################################################
### code chunk number 2: ex8a4.Rnw:27-30
###################################################
require(gstat)
require(sp)
require(lattice)


###################################################
### code chunk number 3: ex8a4.Rnw:125-305
###################################################
ssa.matchv <- function(s.init, s.area,
                   s.vm, s.vm.nc=15, s.sep.pow=2,
                   s.ss.wt=0.5, s.sep.np=NA,
                   s.fix.pts=NA, s.decay=256,
                   s.alpha=0.99, s.prob=0.8,
                   eq.steps=200,
                   max.steps=NA, s.prec=NA) {
  
  # fitness function
  #   argument: current point configuration
  #   implicit: discretized model 'vm.l', cutoff 'cut', bin width 'bin.w'
  #     cutoff is fixed because automatic cutoff will vary as points move
  #             opt.wt is the proportional weight for ss.vgm
  #
  # weights vgm difference by number of point-pairs
  #   and inverse distance squared
  v.diff <- function(pts) {
    v <- variogram(z ~ 1, loc=pts, cutoff=cut, width=bin.w)
    # compare the semivariances
    ss.vgm <- sum((((v$gamma - vm.l$gamma)^2 * v$np) /    
                   v$dist^s.sep.pow))/sum(v$np)
    # compare the point-pair distribution, only for requested bins
    ss.np <- ifelse(is.logical(s.sep.np), 0, 
                    sum(((s.sep.np - v$np)[1:length(s.sep.np)])^2))
    value <- opt.wt*ss.vgm + (1-opt.wt)*ss.np
    return(value)
  }
  
  # helper function: compute initial temperature
    metr.temp.init <- function(n.steps) {
      s <- s.init # initial `fitness' in environ
      sum.fit <- 0; i <- 0 
      while (i < n.steps) {
        pt <- sample(1:n.pts, 1)
        pt.old <- coordinates(scheme)[pt,]
        s.try <- NA
        while (is.logical(s.try)) {
          xnew <- pt.old[1] + runif(1, min=-s.max.shift/2,
                                    max=s.max.shift/2)
          ynew <- pt.old[2] + runif(1, min=-s.max.shift/2,
                                    max=s.max.shift/2)
          pt.new <- data.frame(x = xnew, y = ynew)
          coordinates(pt.new) <- c(1,2)
          # over() requires identical CRS, note order
          proj4string(pt.new) <- proj4string(s.area)
          if (!is.na(over(pt.new, s.area))) s.try <- s
        }
        s.try@coords[pt,] <- c(xnew, ynew)
        # fitness function in environ
        if (is.logical(s.fix.pts)) pts <- s.try
        else pts <- rbind(s.fix.pts, s.try)
        # check new fitness, only use if worse
        fit.new <- v.diff(pts)
        if (fit.new > fitness) {
          sum.fit <- sum.fit + fit.new
          i <- i+1 }
      } # note change of sign to ensure negative sum
      return((fitness-(sum.fit/n.steps))/log(s.prob))
    }

    ## main body
    ## 0 - check and adjust arguments if necessary

    ## 1 - setup  
    # 1.1 - compute maximum distance across bbox
    max.dist <- sqrt(diff(bbox(s.area)[1,])^2
                     + diff(bbox(s.area)[2,])^2)
    n.pts <- length(s.init$z)

    # 1.2 - initialize variogram matching, fixed cutoff
  cut <- max.dist/3
  bin.w <- cut/s.vm.nc
  v <- variogram(z ~ 1, loc=scheme, cutoff=cut, width=bin.w)
  vm.l <- variogramLine(object=s.vm, dist_vector=v$dist)
  
    # 1.2a - normalize the two parts of the objective function
    #   use the initial values as rough guides to the relative magnitude
  m.1 <- s.ss.wt*sum((((v$gamma - vm.l$gamma)^2 * v$np) /    
                   v$dist^s.sep.pow))/sum(v$np)
  m.2 <- (1-s.ss.wt)*
    ifelse(is.logical(s.sep.np), 0, 
           sum((s.sep.np - v$np)[1:length(s.sep.np)])^2)
    # opt.wt is the proportional weight for ss.vgm
  opt.wt <- ifelse(m.2==0, 1, m.2/(m.1+m.2))
  rm(m.1, m.2)

    # 1.3 - initialize point set
    if (is.logical(s.fix.pts)) pts <- s.init
    else pts <- rbind(s.fix.pts, s.init)
    fitness <- v.diff(pts)  # initial fitness
    
    # 1.4 - if precision was not specified, set as 3rd significant
    #     figure of initial fitness
    i <- 0
    while (is.na(s.prec)) {
      if (fitness%/%(10^-i) != 0) s.prec=i+4 else i <- i + 1
    }
    

    # 1.5 - maximum shift depends on point density
    #       this is an empirical value
    s.max.shift <- 2*max.dist/sqrt(n.pts)
  
    # 1.6 - initialize temperature
    metr.temp <- 
      ret.metr.temp.init <-
      metr.temp.init(128); # this is enough for a rough guess

    # 1.7 - set up to record evolution of fits
    fits <- matrix(nrow = 0, ncol=4)
    colnames(fits) <- c("actual","proposed","max.shift","temperature")
    this.step <- 0; equal.steps <- 0;
    s <- s.init;
      
    ## 2 - main loop
    # 2.1 get a proposed new point in the study area
    repeat {
      max.shift <- s.max.shift * exp(-this.step/s.decay)
      metr.temp <- metr.temp * s.alpha

      pt <- sample(1:n.pts, 1)
      pt.old <- coordinates(s)[pt,]
      repeat {
        xnew <- round(pt.old[1] + runif(1, min=-max.shift,
                                       max=max.shift),prec)
        ynew <- round(pt.old[2] + runif(1, min=-max.shift,
                                        max=max.shift),prec)
        pt.new <- data.frame(x = xnew, y = ynew)
        coordinates(pt.new) <- ~ x + y
        proj4string(pt.new) <- proj4string(p)
        if (!is.na(over(pt.new, p))) break
      } # end repeat get a point
      
        # 2.2 increment step when we have a proposed point
      this.step <- this.step + 1;
      s.try <- s;
        # update the selected point's coordinates
      s.try@coords[pt,] <- c(xnew, ynew);
        # 2.3 new fitness, to the desired precision
      if (is.logical(s.fix.pts)) pts <- s.try
      else pts <- rbind(s.fix.pts, s.try)
      fitness.new <-  round(v.diff(pts), s.prec);

        # 2.4 acceptance criteria; maybe update scheme
        if (fitness.new < fitness) {
            s <- s.try;
            fitness <- fitness.new;
            equal.steps <- 0
        } else if (runif(1) < (metr.temp
                               * exp(fitness - fitness.new))) {
                s <- s.try;
                fitness <- fitness.new;
                equal.steps <- 0
        } else { equal.steps <- (equal.steps + 1) }

      # 2.5 save record of fits
      fits <- rbind(fits, c(fitness
                            , fitness.new,
                            max.shift, metr.temp))

      # 2.6 check stopping criteria
      if (equal.steps >= eq.steps) break;
      if (!is.na(max.steps) & (this.step >= max.steps)) break;
    } # end repeat main loop
      
    # 3 - plot the fitness vs. step
    plot(1:length(fits[,1]), fits[,"actual"],
         xlab="Step", ylab="Fitness function", type="l",
         main="Fitness function vs. step",
         sub="Black: actual; red: proposed",
         ylim=c(min(fits[,1]), max(fits[,2])))
      lines(1:length(fits[,1]), fits[,"proposed"],
            type="l", col="red", lty=2)

    # 4 - return the final sampling scheme
    #     and the computed annealing parameters
    return(list(scheme=s,
                metr.temp=ret.metr.temp.init,
                s.prec=s.prec))
  }


###################################################
### code chunk number 4: ex8a4.Rnw:314-326
###################################################
p <- SpatialPolygons(
                     list(
                          Polygons(
                                   list(
                                        Polygon(
                                                matrix(
                                                       c(0, 0, 0, 1, 1, 1, 1, 0, 0, 0),
                                                       byrow = T, nrow = 5,
                                                       ncol = 2))), ID = 1)))
p <- SpatialPolygonsDataFrame(p, data = data.frame(id = 1))
max.dist <- sqrt(2)
prec <- 3


###################################################
### code chunk number 5: ex8a4.Rnw:337-345
###################################################
n.pts <- 80
prob = 0.5
set.seed(25)
scheme <- SpatialPointsDataFrame(
                                 spsample(p, n = n.pts,
                                          type = "random"),
                                 data.frame(z = rep(rbinom(n=n.pts, size=1, prob=prob))))
scheme@coords <- round(scheme@coords, prec)


###################################################
### code chunk number 6: ex8a4.Rnw:350-351
###################################################
spplot(scheme, col.regions=c("red","green"), key.space="", scales=list(draw=T))


###################################################
### code chunk number 7: ex8a4.Rnw:366-368
###################################################
(sill <- (prob)*(1-prob))
vm <- vgm(psill = sill, model = "Exp", range = 0.075, 0)


###################################################
### code chunk number 8: ex8a4.Rnw:375-377
###################################################
v <- variogram(z ~ 1, loc=scheme)
plot(v, model=vm, plot.numbers=T, pch=20, type="b")


###################################################
### code chunk number 9: ex8a4.Rnw:391-399
###################################################
vl <- variogramLine(object=vm, dist_vector=v$dist)
plot(v$gamma ~ v$dist, type="b", xlab="separation", ylab="semivariance", col="red", ylim=c(min(v$gamma, vl$gamma), max(v$gamma, vl$gamma)))
lines(vl$gamma ~ v$dist, type="b", col="darkgreen")
for (i in 1:length(v$dist)) {
  lines(x=c(v$dist[i], v$dist[i]), y=c(v$gamma[i], vl$gamma[i]), lty=2, col="darkgray")
  text(v$dist[i], v$gamma[i], v$np[i], pos=4)
}
legend("bottomright", c("empirical","model","difference"), lty=c(1,1,2), col=c("red","darkgreen","darkgray"))


###################################################
### code chunk number 10: ex8a4.Rnw:407-410
###################################################
time <- system.time(
                    tmp <- ssa.matchv(s.init=scheme, s.area=p, s.vm=vm, s.prob=0.95, s.alpha=0.99, s.decay=1024, eq.steps=200)
            )


###################################################
### code chunk number 11: show.ssa.vm.results
###################################################
print(time)
print(paste("Precision of fitness matching:", tmp[["s.prec"]],
                "decimal places"))
print(paste("Initial Metropolis temperature:",
                round(tmp[["metr.temp"]], 4)))
s.final <- tmp[["scheme"]]


###################################################
### code chunk number 12: side2side (eval = FALSE)
###################################################
## p1 <- spplot(scheme, col.regions=c("red","green"), key.space="", scales=list(draw=T), main="Initial")
## p2 <- spplot(s.final, col.regions=c("red","green"), key.space="", scales=list(draw=T), main="Final")
## print(p1, split=c(1,1,2,1), more=T)
## print(p2, split=c(2,1,2,1), more=F)


###################################################
### code chunk number 13: ex8a4.Rnw:435-436
###################################################
p1 <- spplot(scheme, col.regions=c("red","green"), key.space="", scales=list(draw=T), main="Initial")
p2 <- spplot(s.final, col.regions=c("red","green"), key.space="", scales=list(draw=T), main="Final")
print(p1, split=c(1,1,2,1), more=T)
print(p2, split=c(2,1,2,1), more=F)


###################################################
### code chunk number 14: show.match.vgm.final (eval = FALSE)
###################################################
## v <- variogram(z ~ 1, s.final)
## vl <- variogramLine(object=vm, dist_vector=v[,"dist"])
## plot(v$gamma ~ v$dist, type="b", xlab="separation", ylab="semivariance", col="red", ylim=c(min(v$gamma, vl$gamma), max(v$gamma, vl$gamma)))
## lines(vl$gamma ~ v$dist, type="b", col="darkgreen")
## for (i in 1:length(v$dist)) {
##   lines(x=c(v$dist[i], v$dist[i]), y=c(v$gamma[i], vl$gamma[i]), lty=2, col="darkgray")
##   text(v$dist[i], v$gamma[i], v$np[i], pos=4)
## }
## legend("bottomright", c("empirical","model","difference"), lty=c(1,1,2), col=c("red","darkgreen","darkgray"))


###################################################
### code chunk number 15: ex8a4.Rnw:459-460
###################################################
v <- variogram(z ~ 1, s.final)
vl <- variogramLine(object=vm, dist_vector=v[,"dist"])
plot(v$gamma ~ v$dist, type="b", xlab="separation", ylab="semivariance", col="red", ylim=c(min(v$gamma, vl$gamma), max(v$gamma, vl$gamma)))
lines(vl$gamma ~ v$dist, type="b", col="darkgreen")
for (i in 1:length(v$dist)) {
  lines(x=c(v$dist[i], v$dist[i]), y=c(v$gamma[i], vl$gamma[i]), lty=2, col="darkgray")
  text(v$dist[i], v$gamma[i], v$np[i], pos=4)
}
legend("bottomright", c("empirical","model","difference"), lty=c(1,1,2), col=c("red","darkgreen","darkgray"))


###################################################
### code chunk number 16: ex8a4.Rnw:481-488
###################################################
vl.np <- c(60,80,120)
time <- system.time(
                    tmp <- ssa.matchv(
                                      s.init=scheme, s.area=p, s.vm=vm, s.sep.np=vl.np, 
                                      s.ss.wt=0.8,
                                      s.prob=0.95, s.alpha=0.995, s.decay=1024, eq.steps=200)
            )


###################################################
### code chunk number 17: ex8a4.Rnw:492-493
###################################################
print(time)
print(paste("Precision of fitness matching:", tmp[["s.prec"]],
                "decimal places"))
print(paste("Initial Metropolis temperature:",
                round(tmp[["metr.temp"]], 4)))
s.final <- tmp[["scheme"]]


###################################################
### code chunk number 18: ex8a4.Rnw:501-502
###################################################
p1 <- spplot(scheme, col.regions=c("red","green"), key.space="", scales=list(draw=T), main="Initial")
p2 <- spplot(s.final, col.regions=c("red","green"), key.space="", scales=list(draw=T), main="Final")
print(p1, split=c(1,1,2,1), more=T)
print(p2, split=c(2,1,2,1), more=F)


###################################################
### code chunk number 19: ex8a4.Rnw:514-515
###################################################
v <- variogram(z ~ 1, s.final)
vl <- variogramLine(object=vm, dist_vector=v[,"dist"])
plot(v$gamma ~ v$dist, type="b", xlab="separation", ylab="semivariance", col="red", ylim=c(min(v$gamma, vl$gamma), max(v$gamma, vl$gamma)))
lines(vl$gamma ~ v$dist, type="b", col="darkgreen")
for (i in 1:length(v$dist)) {
  lines(x=c(v$dist[i], v$dist[i]), y=c(v$gamma[i], vl$gamma[i]), lty=2, col="darkgray")
  text(v$dist[i], v$gamma[i], v$np[i], pos=4)
}
legend("bottomright", c("empirical","model","difference"), lty=c(1,1,2), col=c("red","darkgreen","darkgray"))


###################################################
### code chunk number 20: cf.np
###################################################
tmp <- rbind(t1 <- v$np[1:length(vl.np)],
             vl.np, t1-vl.np)
rownames(tmp) <- c("actual","expected","difference")
colnames(tmp) <- round(vl$dist[1:length(vl.np)],2)
print(tmp)
rm(tmp,t1)


###################################################
### code chunk number 21: ex8a4.Rnw:564-728
###################################################
ssa.matchnp <- function(s.init, s.grid,
                   s.vm.nc=15,
                   s.sep.np=NA,
                   s.fix.pts=NA, s.decay=256,
                   s.alpha=0.99, s.prob=0.8,
                   eq.steps=200,
                   max.steps=NA, s.prec=NA) {
  
  # fitness function
  #   argument: current point configuration
  #   implicit: cutoff 'cut', bin width 'bin.w'
  #     cutoff is fixed because automatic cutoff will vary as points move
  #             opt.wt is the proportional weight for ss.vgm
  v.np.diff <- function(pts) {
    v <- variogram(z ~ 1, loc=pts, cutoff=cut, width=bin.w)
    # compare the point-pair distribution, only for requested bins
    return(sum(((s.sep.np - v$np)[1:length(s.sep.np)])^2))
  }
  
  # helper function: compute initial temperature
    metr.temp.init <- function(n.steps) {
      s <- s.init # initial `fitness' in environ
      sum.fit <- 0; i <- 0 
      while (i < n.steps) {
        pt <- sample(1:n.pts, 1)
        pt.old <- coordinates(scheme)[pt,]
        s.try <- NA
        while (is.logical(s.try)) {
          xnew <- pt.old[1] + runif(1, min=-s.max.shift/2,
                                    max=s.max.shift/2)
          ynew <- pt.old[2] + runif(1, min=-s.max.shift/2,
                                    max=s.max.shift/2)
          pt.new <- data.frame(x = xnew, y = ynew)
          coordinates(pt.new) <- c(1,2)
          # over() requires identical CRS, note order
          proj4string(pt.new) <- proj4string(s.grid)
          if (!is.na(over(pt.new, s.grid))) s.try <- s
        }
        s.try@coords[pt,] <- c(xnew, ynew)
        # fitness function in environ
        if (is.logical(s.fix.pts)) pts <- s.try
        else pts <- rbind(s.fix.pts, s.try)
        # check new fitness, only use if worse
        fit.new <- v.np.diff(pts)
        if (fit.new > fitness) {
          sum.fit <- sum.fit + fit.new
          i <- i+1 }
      } # note change of sign to ensure negative sum
      return((fitness-(sum.fit/n.steps))/log(s.prob))
    }

    ### main body
    ## 1 - setup  
    # 1.1 - compute maximum distance across bbox
    max.dist <- sqrt(diff(bbox(s.grid)[1,])^2
                     + diff(bbox(s.grid)[2,])^2)
    n.pts <- length(s.init$z)

    # 1.2 - initialize point set
    if (is.logical(s.fix.pts)) pts <- s.init
    else pts <- rbind(s.fix.pts, s.init)
  
    # 1.3 - initialize variogram matching, fixed cutoff
    cut <- max.dist/3
    bin.w <- cut/s.vm.nc
    v <- variogram(z ~ 1, loc=pts, cutoff=cut, width=bin.w)
  
    # initial fitness
    fitness <- v.np.diff(pts)
    
    # 1.4 - if precision was not specified, set as 3rd significant
    #     figure of initial fitness
    i <- 0
    while (is.na(s.prec)) {
      if (fitness%/%(10^-i) != 0) s.prec=i+4 else i <- i + 1
    }
    
    # 1.5 - maximum shift depends on point density
    #       this is an empirical value
    s.max.shift <- 2*max.dist/sqrt(n.pts)
  
    # 1.6 - initialize temperature
    metr.temp <- 
      ret.metr.temp.init <-
      metr.temp.init(128); # this is enough for a rough guess

    # 1.7 - set up to record evolution of fits
    fits <- matrix(nrow = 0, ncol=4)
    colnames(fits) <- c("actual","proposed","max.shift","temperature")
    this.step <- 0; equal.steps <- 0;
    s <- s.init;
      
    ## 2 - main loop
    repeat {
      max.shift <- s.max.shift * exp(-this.step/s.decay)
      metr.temp <- metr.temp * s.alpha

      pt <- sample(1:n.pts, 1)
      pt.old <- coordinates(s)[pt,]
      ## 2.1 get a proposed new point in the study area
      i <- 1
      repeat { 
        xnew <- round(pt.old[1] + runif(1, min=-max.shift/2,
                                       max=max.shift/2),prec)
        ynew <- round(pt.old[2] + runif(1, min=-max.shift/2,
                                        max=max.shift/2),prec)
        pt.new <- data.frame(x = xnew, y = ynew)
        coordinates(pt.new) <- ~ x + y
        proj4string(pt.new) <- proj4string(p)
        if (!is.na(over(pt.new, s.grid))) break
        else { i <- i + 1
               if (i > max.steps) {
                   stop(paste("Can't find a new point in", max.steps,"steps"))
               }
           }
      } # end repeat get a point

       ## 2.2 increment step when we have a proposed point
      this.step <- this.step + 1;
      s.try <- s;
       ## update the selected point's coordinates
      s.try@coords[pt,] <- c(xnew, ynew);
       ## 2.3 new fitness, to the desired precision
      if (is.logical(s.fix.pts)) pts <- s.try
      else pts <- rbind(s.fix.pts, s.try)
      fitness.new <-  round(v.np.diff(pts), s.prec);

        # 2.4 acceptance criteria; maybe update scheme
        if (fitness.new < fitness) {
            s <- s.try;
            fitness <- fitness.new;
            equal.steps <- 0
        } else if (runif(1) < (metr.temp
                               * exp(fitness - fitness.new))) {
                s <- s.try;
                fitness <- fitness.new;
                equal.steps <- 0
        } else { equal.steps <- (equal.steps + 1) }

      # 2.5 save record of fits
      fits <- rbind(fits, c(fitness
                            , fitness.new,
                            max.shift, metr.temp))

      # 2.6 check stopping criteria
      if (equal.steps >= eq.steps) break;
      if (!is.na(max.steps) & (this.step >= max.steps)) break;
    } # end repeat main loop
      
    # 3 - plot the fitness vs. step
    plot(1:length(fits[,1]), fits[,"actual"],
         xlab="Step", ylab="Fitness function", type="l",
         main="Fitness function vs. step",
         sub="Black: actual; red: proposed",
         ylim=c(min(fits[,1]), max(fits[,2])))
      lines(1:length(fits[,1]), fits[,"proposed"],
            type="l", col="red", lty=2)

    # 4 - return the final sampling scheme
    #     and the computed annealing parameters
    return(list(scheme=s,
                metr.temp=ret.metr.temp.init,
                s.prec=s.prec))
  }


###################################################
### code chunk number 22: ex8a4.Rnw:737-744
###################################################
data(meuse)
coordinates(meuse) <- ~x+y
meuse$lzn <-log(meuse$zinc)
summary(meuse$lzn)
v <- variogram(lzn ~ 1, loc=meuse)
(vm <- fit.variogram(v, vgm(0.4, "Sph", 1000, 0.2)))
plot(v, model=vm, pl=T)


###################################################
### code chunk number 23: ex8a4.Rnw:752-761
###################################################
data(meuse.grid)
coordinates(meuse.grid) <- ~x+y
gridded(meuse.grid) <- T
## spsample uses the bboc for a spGrid, actual pixels for spPixels
##  so do not promote to a full grid, which it isn't anyway
## fullgrid(meuse.grid) <- T
set.seed(99)
k.sim<-krige(lzn ~1, loc=meuse, newdata=meuse.grid, model=vm, beta=0, dummy=T, nsim=1, nmax=120)
spplot(k.sim, col.regions=bpy.colors(64))


###################################################
### code chunk number 24: ex8a4.Rnw:772-781
###################################################
set.seed(7523)
scheme <- spsample(meuse.grid, 100, type="random")
tmp <- over(scheme, k.sim)
summary(tmp)
# variable must be named 'z'
scheme.df <- SpatialPointsDataFrame(scheme, 
                                    data=data.frame(z=tmp$sim1))
spplot(scheme.df, zcol="z", col.regions=bpy.colors(64),
       key.space="right")


###################################################
### code chunk number 25: ex8a4.Rnw:790-798
###################################################
v.scheme <- variogram(z ~ 1, scheme.df)
plot(v.scheme$gamma ~ v.scheme$dist, type="b", col="red", xlab="Separation", ylab="semivariance", main="Variograms, log(Zn)")
text(v.scheme$dist, v.scheme$gamma, v.scheme$np, pos=2, col="red")
lines(v$gamma ~ v$dist, type="b", col="darkgreen")
text(v$dist, v$gamma, v$np, pos=3, col="darkgreen")
lines(variogramLine(vm, maxdist=max(v$dist)), col="blue")
grid()
legend("bottomright",c("initial scheme", "simulated field", "model"), lty=1, col=c("red","darkgreen","blue"))


###################################################
### code chunk number 26: ex8a4.Rnw:812-823
###################################################
vl.np <- c(60,80,120, 240, 280)
# debug(ssa.matchnp)
# undebug(ssa.matchnp)
time <- system.time(
                    tmp <- ssa.matchnp(
                                      s.init=scheme.df,
                                      s.grid=meuse.grid,
                                      s.sep.np=vl.np,
                                      s.prob=0.95, s.alpha=0.995,
                                      s.decay=1024, eq.steps=200, max.steps=2000)
            )


###################################################
### code chunk number 27: ex8a4.Rnw:828-829
###################################################
print(time)
print(paste("Precision of fitness matching:", tmp[["s.prec"]],
                "decimal places"))
print(paste("Initial Metropolis temperature:",
                round(tmp[["metr.temp"]], 4)))
s.final <- tmp[["scheme"]]


###################################################
### code chunk number 28: ex8a4.Rnw:835-838
###################################################
summary(s.final$z)
s.final$z <- over(s.final, k.sim)$sim1
summary(s.final$z)


###################################################
### code chunk number 29: ex8a4.Rnw:844-846
###################################################
str(s.final)
spplot(s.final, col.regions=bpy.colors(64), key.space="right")


###################################################
### code chunk number 30: ex8a4.Rnw:852-853
###################################################
v <- variogram(z ~ 1, s.final)
vl <- variogramLine(object=vm, dist_vector=v[,"dist"])
plot(v$gamma ~ v$dist, type="b", xlab="separation", ylab="semivariance", col="red", ylim=c(min(v$gamma, vl$gamma), max(v$gamma, vl$gamma)))
lines(vl$gamma ~ v$dist, type="b", col="darkgreen")
for (i in 1:length(v$dist)) {
  lines(x=c(v$dist[i], v$dist[i]), y=c(v$gamma[i], vl$gamma[i]), lty=2, col="darkgray")
  text(v$dist[i], v$gamma[i], v$np[i], pos=4)
}
legend("bottomright", c("empirical","model","difference"), lty=c(1,1,2), col=c("red","darkgreen","darkgray"))


###################################################
### code chunk number 31: ex8a4.Rnw:867-868
###################################################
tmp <- rbind(t1 <- v$np[1:length(vl.np)],
             vl.np, t1-vl.np)
rownames(tmp) <- c("actual","expected","difference")
colnames(tmp) <- round(vl$dist[1:length(vl.np)],2)
print(tmp)
rm(tmp,t1)


