ssa.ok <- function(s.init, s.area, s.res,
                   s.model, s.fix.pts=NA, s.f=mean, s.decay=256,
                   s.max.shift = sqrt(2)/4,
                   s.alpha=0.99, s.prob=0.8,
                   eq.steps=200,
                   max.steps=NA, s.prec=NA) {

  
  # helper function: compute fitness by kriging variance
  # interpolation `grid' and variogram model `s.model'
  #   are in callers environment
  # data field must be named `z'
  obj.k <- function(pts, f) {
      k <- krige(z ~ 1, loc=pts, model=s.model,
                 newdata=grid, debug.level=0)
      return(f(k$var1.var))
    }
  
  # helper function: compute initial temperature
    metr.temp.init <- function(n.steps) {
      s <- s.init # initial `fitness' in environ
      max.shift <- max.dist*sqrt(2)/4
      sum.fit <- 0; i <- 0 
      while (i < n.steps) {
        pt <- sample(1:n.pts, 1)
        pt.old <- coordinates(scheme)[pt,]
        s.try <- NA
        while (is.logical(s.try)) {
          xnew <- pt.old[1] + runif(1, min=-max.shift/2,
                                    max=max.shift/2)
          ynew <- pt.old[2] + runif(1, min=-max.shift/2,
                                    max=max.shift/2)
          pt.new <- data.frame(x = xnew, y = ynew)
          coordinates(pt.new) <- c(1,2)
          # over() requires identical CRS, note order
          proj4string(pt.new) <- proj4string(s.area)
          if (!is.na(over(pt.new, s.area))) s.try <- s
        }
        s.try@coords[pt,] <- c(xnew, ynew)
        # fitness function in environ
        if (is.logical(s.fix.pts)) pts <- s.try
        else pts <- rbind(s.fix.pts, s.try)
        # check new fitness, only use if worse
        fit.new <- obj.k(pts, s.f)
        if (fit.new > fitness) {
          sum.fit <- sum.fit + fit.new
          i <- i+1 }
      } # note change of sign to ensure negative sum
      return((fitness-(sum.fit/n.steps))/log(s.prob))
    }

    ## main body
    ## 1 - setup
    # 1.1 - set up maximum distance
    max.dist <- sqrt(((bbox(s.area)[1,"max"]
                       - bbox(s.area)[1,"min"])^2)
                     + ((bbox(s.area)[2,"max"]
                         - bbox(s.area)[2,"min"])^2))
    n.pts <- length(s.init$z)

    # 1.2 - set up interpolation grid -- may not be full
    #       so leave as SpatialPoints, do not promote to SpatialGrid
  grid <- spsample(s.area, n=s.res^2, type="regular")
  if (is.logical(s.fix.pts)) pts <- s.init
  else pts <- rbind(s.fix.pts, s.init)
  fitness <- obj.k(pts, s.f)  # initial fitness
    
    # 1.3 - if precision was not specified, set as 3rd significant
    #     figure of initial fitness
    i <- 0
    while (is.na(s.prec)) {
      if (fitness%/%(10^-i) != 0) s.prec=i+4 else i <- i + 1
    }
    
    # 1.4 - initialize temperature
    metr.temp <- 
      ret.metr.temp.init <-
      metr.temp.init(128); # this is enough for a rough guess

    # 1.5 - set up to record evolution of fits
    fits <- matrix(nrow = 0, ncol=4)
    colnames(fits) <- c("actual","proposed","max.shift","temperature")
    this.step <- 0; equal.steps <- 0;
    s <- s.init;
    
    ## 2 - main loop
    # 2.1 get a proposed new point in the study area
    repeat {
      max.shift <- (max.dist*s.max.shift)*
        exp(-this.step/s.decay)
      metr.temp <- metr.temp * s.alpha

      pt <- sample(1:n.pts, 1)
      pt.old <- coordinates(s)[pt,]
      repeat {
        xnew <- round(pt.old[1] + runif(1, min=-max.shift,
                                       max=max.shift),prec)
        ynew <- round(pt.old[2] + runif(1, min=-max.shift,
                                        max=max.shift),prec)
        pt.new <- data.frame(x = xnew, y = ynew)
        coordinates(pt.new) <- ~ x + y
        proj4string(pt.new) <- proj4string(p)
        if (!is.na(over(pt.new, p))) break
      }

        # 2.2 increment step when we have a proposed point
      this.step <- this.step + 1;
      s.try <- s;
        # update the selected point's coordinates
      s.try@coords[pt,] <- c(xnew, ynew);
        # 2.3 new fitness, to the desired precision
      if (is.logical(s.fix.pts)) pts <- s.try
      else pts <- rbind(s.fix.pts, s.try)
      fitness.new <-  round(obj.k(pts, s.f), s.prec);

        # 2.4 acceptance criteria; maybe update scheme
        if (fitness.new < fitness) {
            s <- s.try;
            fitness <- fitness.new;
            equal.steps <- 0
        } else if (runif(1) < (metr.temp
                               * exp(fitness - fitness.new))) {
                s <- s.try;
                fitness <- fitness.new;
                equal.steps <- 0
        } else { equal.steps <- (equal.steps + 1) }

      # 2.5 save record of fits
      fits <- rbind(fits, c(fitness, fitness.new,
                            max.shift, metr.temp))

      # 2.6 check stopping criteria
      if (equal.steps >= eq.steps) break;
      if (!is.na(max.steps) & (this.step >= max.steps)) break;
    }
      
    # 3 - plot the fitness vs. step
    plot(1:length(fits[,1]), fits[,"actual"],
         xlab="Step", ylab="Fitness function", type="l",
         main="Fitness function vs. step",
         sub="Black: actual; red: proposed",
         ylim=c(min(fits[,1]), max(fits[,2])))
      lines(1:length(fits[,1]), fits[,"proposed"],
            type="l", col="red", lty=2)
    
    # 4 - return the final sampling scheme
    #     and the computed annealing parameters
    return(list(scheme=s, metr.temp=ret.metr.temp.init, s.prec=s.prec))
  }
